@@ -6,6 +6,7 @@ import os
 import json
 import uuid
 import argparse
+import shutil
 import random
 import string
 from datetime import datetime, timezone, timedelta
@@ -28,7 +29,6 @@ import logging
 from pathlib import Path
 import pyte
 from . import host_ip
-from flask_swagger_ui import get_swaggerui_blueprint  # new import
 import yaml
 
 if os.environ.get('PYWEBEXEC_LDAP_SERVER'):
@@ -359,7 +359,8 @@ def parseargs():
     parser.add_argument("-g", "--gencert", action="store_true", help="https server self signed cert")
     parser.add_argument("-T", "--tokenurl", action="store_true", help="generate safe url to access")
     parser.add_argument("action", nargs="?", help="daemon action start/stop/restart/status/shareterm/term",
-                        choices=["start","stop","restart","status","shareterm", "term"])
+                        choices=["start","stop","restart","status","shareterm", "term", "run", "run-para"])
+    parser.add_argument("command", nargs="*", help="command to run")    
 
     args = parser.parse_args()
     if not os.path.exists(CONFDIR):
@@ -379,6 +380,28 @@ def parseargs():
     if args.action == "term":
         COMMAND_STATUS_DIR = f"{os.getcwd()}/{COMMAND_STATUS_DIR}"
         sys.exit(start_term())
+    elif args.action == "run":
+        command = args.command[0]
+        params = args.command[1:]
+        command_id = term_command_id
+        print("Command:", command_id, flush=True)
+        exit_code = run_command("localhost", args.user, command, params, command_id, tty_rows, tty_cols)
+        sys.exit(exit_code)
+    elif args.action == "run-para":
+        runpara = shutil.which("run-para")
+        if not runpara:
+            print("Error: run-para not found, install using pip install run-para", file=sys.stderr)
+            sys.exit(1)
+        args.command = sys.argv[sys.argv.index("run-para")+1:]
+        try:
+            commandindex = args.command.index("--")
+        except ValueError:
+            print("Error: run-para: No command supplied", file=sys.stderr)
+            sys.exit(1)
+        runparams = args.command[0:commandindex]
+        command = args.command[commandindex+1:]
+        result = subprocess.run([runpara, '-n', *runparams, "--", sys.argv[0], "-d", args.dir, "--", "run", *command], bufsize=0)
+        sys.exit(result.returncode)
 
     (hostname, ip) = host_ip.get_host_ip(args.listen)
 
@@ -484,10 +507,14 @@ def run_command(fromip, user, command, params, command_id, rows, cols):
     # app.logger.info(f'{fromip} run_command {command_id} {user}: {command} {params}')
     log_info(fromip, user, f'run_command {command_id}: {command_str(command, params)}')
     start_time = datetime.now(timezone.utc).isoformat()
+    if user:
+        os.environ['PYWEBEXEC_USER'] = user
     update_command_status(command_id, {
         'command': command,
         'params': params,
         'start_time': start_time,
+        'status': 'running',
+        'from': fromip,
         'user': user,
         'cols': cols,
         'rows': rows,
@@ -540,7 +567,8 @@ def run_command(fromip, user, command, params, command_id, rows, cols):
         with open(get_output_file_path(command_id), 'a') as output_file:
             output_file.write(str(e))
         app.logger.error(fromip, user, f'Error running command {command_id}: {e}')
-
+        exit_code = 1
+    return exit_code
 
 def command_str(command, params):
     try:
@@ -604,24 +632,6 @@ def check_processes():
                         'exit_code': -1,
                     })
 
-args = parseargs()
-if args.cert:
-    app.config['SESSION_COOKIE_SECURE'] = True
-app.config['TITLE'] = f"{args.title} API"
-
-# Register Swagger UI blueprint with safe token included in API_URL
-SWAGGER_URL = '/v0/documentation'
-API_URL = '/swagger.yaml'
-swaggerui_blueprint = get_swaggerui_blueprint(
-    SWAGGER_URL,
-    API_URL,
-    config={
-        'app_name': f"{args.title} API",
-        'layout': 'BaseLayout'
-    }
-)
-app.register_blueprint(swaggerui_blueprint, url_prefix=SWAGGER_URL)
-
 def log_info(fromip, user, message):
     app.logger.info(f"{user} {fromip}: {message}")
 
@@ -631,6 +641,14 @@ def log_error(fromip, user, message):
 def log_request(message):
     log_info(request.remote_addr, session.get('username', '-'), message)
 
+
+args = parseargs()
+if args.cert:
+    app.config['SESSION_COOKIE_SECURE'] = True
+app.config['TITLE'] = f"{args.title} API"
+
+
+
 def get_executable(cmd):
     if os.path.isfile(cmd) and os.access(cmd, os.X_OK):
         help_file = f"{cmd}.help"
@@ -639,7 +657,7 @@ def get_executable(cmd):
             with open(help_file, 'r') as hf:
                 help_text = hf.read()
         schema_file = f"{cmd}.schema.yaml"
-        schema = None
+        schema = {}
         if os.path.exists(schema_file):
             with open(schema_file, 'r') as sf:
                 schema = yaml.safe_load(sf)
@@ -649,11 +667,12 @@ def get_executable(cmd):
 def get_executables():
     executables_list = []
     for f in os.listdir('.'):
-            exe = get_executable(f)
-            if exe:
-                executables_list.append(exe)
+        exe = get_executable(f)
+        if exe:
+            executables_list.append(exe)
     return sorted(executables_list, key=lambda x: x["command"])
 
+
 @app.route('/commands/<command_id>/stop', methods=['PATCH'])
 def stop_command(command_id):
     log_request(f"stop_command {command_id}")
@@ -785,9 +804,69 @@ def list_executables():
     return jsonify({"commands": executables_list})
 
 
+    
 @app.route('/commands/<cmd>', methods=['POST'])
 def run_dynamic_command(cmd):
     # Validate that 'cmd' is an executable in the current directory
+
+    def parse_params(cmd, data_params):
+        if not data_params:
+            return []
+        # Convert received parameters to exec args.
+        # If schema defined for each para, value in data:
+        # - if value is True → --<key>
+        # - if value is a string → --<param> value
+        # - if value is an array → --<param> value1 value2 ...
+        # - if value is False → "" (omit)
+        # schema_options:
+        #   separator: " " (default) or "=" is the separator between --param and value
+        #   noprefix_params: ["param1", "param2"] or ["*"] to omit --param prefix
+        #   convert_params: {"param1": "param2"} to convert param1 to param2
+        exe = get_executable(cmd)
+        separator = exe.get("schema", {}).get("schema_options", {}).get("separator", " ")
+        noprefix = exe.get("schema", {}).get("schema_options", {}).get("noprefix_params", {})
+        convert_params = exe.get("schema", {}).get("schema_options", {}).get("convert_params", {})
+        schema_params = exe.get("schema", {}).get("properties", {})
+        if isinstance(data_params, dict) and not schema_params:
+            return None
+        if isinstance(data_params, dict):
+            params = ""
+            for param in schema_params.keys():
+                if not param in data_params:
+                    continue
+                value = data_params[param]
+                if value is None:
+                    continue
+                prefix = ""
+                if param in convert_params:
+                    param = convert_params[param]
+                    prefix = param
+                    if param in ['--', '', None]:
+                        separator = ' '
+                elif "*" in noprefix or param in noprefix:
+                    separator = ""
+                else:
+                    prefix = f"--{param}"
+                if isinstance(value, bool):
+                    if value:
+                        params += f"{prefix} "
+                    continue
+                params += f"{prefix}{separator}"
+                if isinstance(value, list):
+                    params += " ".join(value)
+                else:
+                    params += str(value)
+                params += " "
+        else:
+            params = data_params
+        if isinstance(params, str):
+            params = [params]
+        try:
+            params = shlex.split(' '.join(params)) if isinstance(params, list) else []
+        except Exception as e:
+            return None
+        return params
+
     cmd_path = os.path.join(".", os.path.basename(cmd))
     if not os.path.isfile(cmd_path) or not os.access(cmd_path, os.X_OK):
         return jsonify({'error': 'Command not found or not executable'}), 400
@@ -795,54 +874,9 @@ def run_dynamic_command(cmd):
         data = request.json
     except Exception as e:
         data = {}
-
-    # Convert received parameters to exec args.
-    # If schema defined for each para, value in data:
-    # - if value is True → --<key>
-    # - if value is a string → --<param> value
-    # - if value is an array → --<param> value1 value2 ...
-    # - if value is False → "" (omit)
-    # schema_options:
-    #   separator: " " (default) or "=" is the separator between --param and value
-    #   noprefix_params: ["param1", "param2"] or ["*"] to omit --param prefix
-    #   convert_params: {"param1": "param2"} to convert param1 to param2
-    exe = get_executable(cmd)
-    separator = exe.get("schema", {}).get("schema_options", {}).get("separator", " ")
-    noprefix = exe.get("schema", {}).get("schema_options", {}).get("noprefix_params", {})
-    convert_params = exe.get("schema", {}).get("schema_options", {}).get("convert_params", {})
-    if data.get('params') and isinstance(data['params'], dict):
-        params = ""
-        for param, value in data['params'].items():
-            if value is None:
-                continue
-            prefix = ""
-            if param in convert_params:
-                param = convert_params[param]
-                prefix = param
-                if param in ['--', '', None]:
-                    separator = ' '
-            elif "*" in noprefix or param in noprefix:
-                separator = ""
-            else:
-                prefix = f"--{param}"
-            if isinstance(value, bool):
-                if value:
-                    params += f"{prefix} "
-                continue
-            params += f"{prefix}{separator}"
-            if isinstance(value, list):
-                params += " ".join(value)
-            else:
-                params += str(value)
-            params += " "
-    else:
-        params = data.get("params", [])
-    if isinstance(params, str):
-        params = [params]
-    try:
-        params = shlex.split(' '.join(params)) if isinstance(params, list) else []
-    except Exception as e:
-        return jsonify({'error': str(e)}), 400
+    params = parse_params(cmd, data.get('params'))
+    if params is None:
+        return jsonify({'error': 'Invalid parameters'}), 400
     rows = data.get('rows', tty_rows) or tty_rows
     cols = data.get('cols', tty_cols) or tty_cols
     user = session.get('username', '-')
@@ -870,6 +904,10 @@ def get_command_status(command_id):
 def index():
     return render_template('index.html', title=args.title)
 
+@app.route('/v0/documentation/')
+def swagger_ui():
+    return render_template('swagger_ui.html', title=app.config.get('TITLE', 'PyWebExec API'))
+
 @app.route('/commands', methods=['GET'])
 def list_commands():
     commands = read_commands()
@@ -982,8 +1020,8 @@ def swagger_yaml():
                 "type": "object",
                 "properties": {
                     "params": {"type": "array", "items": {"type": "string"}, "default": []},
-                    "rows": {"type": "integer", "default": tty_rows},
-                    "cols": {"type": "integer", "default": tty_cols}
+                    "rows": {"type": "integer", "description": "tty nb rows", "default": tty_rows},
+                    "cols": {"type": "integer", "description": "tty nb columns", "default": tty_cols}
                 }
             }
             if exe["schema"]:
@@ -1000,7 +1038,7 @@ def swagger_yaml():
                     "parameters": [
                         {
                             "in": "body",
-                            "name": "commandRequest",
+                            "name": "reuqestBody",
                             "schema": cmd_schema
                         }
                     ],
@@ -1019,7 +1057,7 @@ def swagger_yaml():
                 }
             }
         swagger_spec['info']['title'] = app.config.get('TITLE', 'PyWebExec API')
-        swagger_spec_str = yaml.dump(swagger_spec)
+        swagger_spec_str = yaml.dump(swagger_spec, sort_keys=False)
         return Response(swagger_spec_str, mimetype='application/yaml')
     except Exception as e:
         return Response(f"Error reading swagger spec: {e}", status=500)
